#!/bin/bash

if [ $# = 2 ]
 then
  echo "calcul $1"
 else
  echo "Syntaxe : $0 NUM_CALCUL options=$2"
fi 

CALCUL=$1
OPT_QTLMAP=$2

clear

#============================================================
ROOTDIR=$PWD

#============================================================

if [ -z $ACTIVE_QTLMAP ] 
then
  echo " $0 : ACTIVE_QTLMAP - env variable is not defined."
  exit 1;
fi

if [ -z $NSIM ]
then
  echo " $0 : NSIM - env variable is not defined."
  exit 1;
fi

if [ -z $NSIM_SEUIL_QTLMAP ]
then
  echo " $0 : NSIM_SEUIL_QTLMAP - env variable is not defined."
  exit 1;
fi

if [ -z $LONG_CHROMO ]
then
  echo " $0 : LONG_CHROMO - env variable is not defined."
  exit 1;
fi

if [ -z $NB_MARKER ]
then
  echo " $0 : NB_MARKER - env variable is not defined."
  exit 1;
fi

if [ -z $POS_QTL ]
then
  echo "$0 :  POS_QTL - env variable is not defined."
  exit 1;
fi

if [ -z $EFF_QTL ]
then
  echo " $0 : EFF_QTL - env variable is not defined."
  exit 1;
fi

if [ -z $H2 ]
then
  echo " $0 : EFF_QTL - env variable is not defined."
  exit 1;
fi

if [ -z $RATIO_POLQTL ]
then
  echo " $0 : RATIO_POLQTL - env variable is not defined."
  exit 1;
fi

if [ -z $NG ]
then
  echo " $0 : NG - env variable is not defined."
  exit 1;
fi

if [ -z $NMPED ]
then
  echo " $0 : NMPED - env variable is not defined."
  exit 1;
fi

if [ -z $NFMPED ]
then
  echo " $0 : NFMPED - env variable is not defined."
  exit 1;
fi

if [ -z $NDFPED ]
then
  echo " $0 : NDFPED - env variable is not defined."
  exit 1;
fi

if [ -z $NMQTL ]
then
  echo " $0 : NMQTL - env variable is not defined."
  exit 1;
fi

if [ -z $NFMQTL ]
then
  echo " $0 : NFMQTL - env variable is not defined."
  exit 1;
fi

if [ -z $NDFQTL ]
then
  echo " $0 : NDFQTL - env variable is not defined."
  exit 1;
fi

if [ -z $GPU_ID ]
then
  echo " $0 : GPU_ID - env variable is not defined."
  exit 1;
fi


NDESCQTL=$(($NMQTL*NFMQTL*NDFQTL))

#-----------------------------------------------------------------------------------------------------

source ./functions.sh

PWDD=$PWD

echo
echo " ** LOG $LOGWORK"
echo
echo

WORK=NG${NG}_NMPED${NMPED}_NFMPED${NFMPED}_NDFPED${NDFPED}_NMQTL${NMQTL}_NFMQTL${NFMQTL}_NDFQTL${NDFQTL}
mkdir -p $WORK
pushd $WORK > /dev/null

WORK=H2_${H2}_RATIO_POLQTL${RATIO_POLQTL}_LONGCHROMO${LONG_CHROMO}_NMK${NB_MARKER}_POS_QTL${POS_QTL}_EFF_QTL${EFF_QTL}
mkdir -p $WORK
pushd $WORK > /dev/null

LOGWORK=$PWD/log.$$
touch $LOGWORK

PREFIX=NDMIN${QTLMAP_NDMIN}_$NSIM_SEUIL${NSIM_SEUIL_QTLMAP}_STEP${QTLMAP_STEP}_MODELH2_${QTLMAP_H2}_NSIMTHRESH${NSIM_SEUIL_QTLMAP}
RESUME_WORKFLOW=$PWD/RESUME_CALCUL_${PREFIX}_$CALCUL

# Header of resum file
header() {  

RESUM=$1
echo " == INFO DATASET SIMUL == ">> $RESUM
echo "" >> $RESUM

echo "LONG_CHROMO=$LONG_CHROMO">> $RESUM
echo "NB_MARKER=$NB_MARKER">> $RESUM
echo "POS_QTL=$POS_QTL">> $RESUM
echo "EFF_QTL=$EFF_QTL">> $RESUM
echo "H2=$H2">> $RESUM
echo "RATIO_POLQTL=$RATIO_POLQTL">> $RESUM

echo "---------------">> $RESUM
echo "NG=$NG">> $RESUM
echo "NMPED=$NMPED">> $RESUM
echo "NFMPED=$NFMPED">> $RESUM
echo "NDFPED=$NDFPED">> $RESUM
echo "NMQTL=$NMQTL">> $RESUM
echo "NFMQTL=$NFMQTL">> $RESUM
echo "NDFQTL=$NDFQTL">> $RESUM
echo "OPT_QTLMAP=$OPT_QTLMAP">> $RESUM
echo "---------------">> $RESUM
echo "QTLMAP_H2=${QTLMAP_H2}">> $RESUM
echo "QTLMAP_NSIM=${NSIM_SEUIL_QTLMAP}">> $RESUM

echo "" >> $RESUM

echo " == SIMUL == ">> $RESUM
echo "GROUPE_SEED POSIQTL  LRT  MARK_FLK_LEFT  MARK_FLK_RIGHT THRES_0.1000  THRES_0.0500 THRES_0.0100 THRES_0.0050 THRES_0.0027 THRES_0.0001 NB_SIRES_HETERO" >> $RESUM

}

#adding information
add_result() {
RESUM=$1
POSQTL=`grep "The maximum is reached at position(s)" result | awk '{ print $7}'| awk 'BEGIN {FS="("} {print $1}'`
LRT=`grep "Test H0 / H1" result | awk 'BEGIN {FS=":"} {print $2}'`
MARK_FLANKING_LEFT=`grep "flanking marker (qtl:    1) " result | awk 'BEGIN {FS=","} {print $2}'`
MARK_FLANKING_RIGHT=`grep "flanking marker (qtl:    1) " result | awk 'BEGIN {FS=","} {print $3}'`
SEUIL_1=`grep "|   0.1000   |" result | awk 'BEGIN {FS="|"} {print $4}'`
SEUIL_2=`grep "|   0.0500   |" result | awk 'BEGIN {FS="|"} {print $4}'`
SEUIL_3=`grep "|   0.0100   |" result | awk 'BEGIN {FS="|"} {print $4}'`
SEUIL_4=`grep "|   0.0050   |" result | awk 'BEGIN {FS="|"} {print $4}'`
SEUIL_5=`grep "|   0.0027   |" result | awk 'BEGIN {FS="|"} {print $4}'`
SEUIL_6=`grep "|   0.0010   |" result | awk 'BEGIN {FS="|"} {print $4}'`
SEUIL_7=`grep "|   0.0005   |" result | awk 'BEGIN {FS="|"} {print $4}'`
SEUIL_8=`grep "|   0.0001   |" result | awk 'BEGIN {FS="|"} {print $4}'`

echo "${WORK} $POSQTL $LRT $MARK_FLANKING_LEFT $MARK_FLANKING_RIGHT $SEUIL_1 $SEUIL_2 $SEUIL_3 $SEUIL_4 $SEUIL_5 $SEUIL_6 $SEUIL_7 $SEUIL_8 $nbhetero" >> $RESUM
}

if [ $ACTIVE_QTLMAP = "OK" ]
then
if [ ! -e $RESUME_WORKFLOW ]
then
 header $RESUME_WORKFLOW 
fi

fi

#il y a peut etre suffisamment de simulation, on calcul seulement ce qui manque.... 
NSIM_DONE=`ls -d [0-9]* | wc -w`

echo "                     *****  Numner of simulation $NSIM / find ${NSIM_DONE} simulation ******"
NEWNSIM=$(( $NSIM - ${NSIM_DONE} ))


#for ((iGlob = 1; iGlob <= $NEWNSIM; iGlob += 1))
iGlob=1
while [ $iGlob -le $NEWNSIM ]
do 
echo "`date` ############################## $iGlob/$NEWNSIM ########################################"
SEED_PEDIG=$RANDOM
SEED1=$RANDOM
SEED2=$RANDOM
SEED3=$RANDOM
WORK=${SEED_PEDIG}_${SEED1}_${SEED2}_${SEED3}
# si le repertoire existe deja, pas besoin de recalculer cette simulation.....
if [ ! -d $WORK ]
then
echo " work... ** see $PWD/log.$$ "
mkdir -p $WORK
pushd $WORK #> /dev/null
source $PWDD/simul_all.sh > log.$$
VALMENDEL=`grep mendel log.$$ | awk 'BEGIN {FS="="} {print $2}'`
VALENVIR=`grep envir log.$$ | awk 'BEGIN {FS="="} {print $2}'`
OKMENDEL=`echo $VALMENDEL | awk '{ if ( $1 > 0.01 ) { print "OK"}else {print "KO"}}'`
OKENVIR=`echo $VALENVIR | awk '{ if ( $1 > 0.01 ) { print "OK"}else {print "KO"}}'`
OKPERF=`cat perfqtlmap | awk '{print $2}' |  uniq | awk 'END { if (NR == 1 ) {print "KO"} else {print "OK"}}' `
echo "$PWD - $iGlob:mendel=$OKMENDEL,$VALMENDEL envir=$OKENVIR,$VALENVIR"
if [ "$OKMENDEL" = "OK" ] && [ "$OKENVIR" = "OK" ] && [ "$OKPERF" = "OK" ] && [ $nbhetero -gt 0 ]
#then
#if [ "$OKENVIR" = "OK" ] && [ "$OKPERF" = "OK"]
#then
#if [ "$OKPERF" = "OK" ] 
#then
#if [ $nbhetero -gt 0 ]
then
echo "hetero:"$nbhetero
iGlob=$(( ${iGlob} + 1 ))
if [ $ACTIVE_QTLMAP = "OK" ]
then
$QTLMAP p_analyse --calcul=$CALCUL $OPT_QTLMAP --nsim=$NSIM_SEUIL_QTLMAP $OPTIONS_QTLMAP >> log.$$
cp result result_$CALCUL
cp summary summary_$CALCUL
add_result $RESUME_WORKFLOW

clean_qtlmap
fi #activeqtlmap
popd > /dev/null
else
popd
echo "none sires heterozygote...."
rm -rf $WORK > /dev/null
fi #OKENVIR
fi
done

######################################################################################

#### APPEL SEULEMENT QTLMAP (SI PAS ENCORE FAIT) POUR LES SIMUL DEJA CREER AVEC LES SEEDS
if [ $ACTIVE_QTLMAP = "OK" ]
then
iGlob=0
for WORK in `ls -d [0-9]*`
do
if [ `cat $WORK/perfqtlmap | awk '{print $2}' | uniq | wc -l` == "1" ]
then
 echo " ****  $WORK BAD SIMULATION ***** "
 rm -rf $WORK
else
iGlob=$(( ${iGlob} + 1 ))
echo "  $iGlob  check ==>  " $PWD/$WORK 
EXISTE=`cat $RESUME_WORKFLOW | awk 'NF>3 {print $0}'| grep "$WORK"`
if [ -z "$EXISTE" ]
then
pushd $WORK > /dev/null
SEED_PEDIG=`echo "$WORK" | awk 'BEGIN{FS="_"} {print $1}'`
SEED1=`echo "$WORK" | awk 'BEGIN{FS="_"} {print $2}'`
SEED2=`echo "$WORK" | awk 'BEGIN{FS="_"} {print $3}'`
SEED3=`echo "$WORK" | awk 'BEGIN{FS="_"} {print $4}'`
source $PWDD/simul_all.sh > log.$$
panalyseqtlmap_file ${QTLMAP_STEP} ${QTLMAP_NDMIN}
modelqtlmap_file ${QTLMAP_H2}
echo "calcul..:"$CALCUL
$QTLMAP p_analyse --calcul=$CALCUL $OPT_QTLMAP --nsim=$NSIM_SEUIL_QTLMAP >> log.$$
cp result result_$CALCUL
cp summary summary_$CALCUL
add_result $RESUME_WORKFLOW
clean_qtlmap
popd > /dev/null
fi

fi
done
fi

echo
echo
echo "RESUM SIMULATION CALCUL = $CALCUL ==> $RESUME_WORKFLOW "

popd > /dev/null
popd > /dev/null

